import frostypy.core
#from . import density
#from . import porosity
#from . import thermal_conductivity
#from . import thickness

#from .core import *
#from .density import * 
#from .porosity import *
#from .thermal_conductivity import * 
#from .thickness import *



#__all__ = ['core', 'density', 'porosity', 'thermal_conductivity', 'thickness']

#__all__.extend(core.__all__)
#__all__.extend(density.__all__)
#__all__.extend(porosity.__all__)
#__all__.extend(thermal_conductivity.__all__)
#__all__.extend(thickness.__all__)



#submodules = (core, density, porosity, thermal_conductivity, thickness)

__version__ = '0.0.15'

def main():
    """Entry point for the application script"""
    print("Call your main application code here")



# -*- coding: utf-8 -*-
"""
.. contents:: :local:

Thermal Conductivity
--------------------
.. autofunction:: Lee
.. autofunction:: Ostin
.. autofunction:: Yonko
"""

__all__ = ['Lee', 'Ostin', 'Yonko']

def Lee(rho_f):
    r'''Calculates the frost thermal conductivity, k_f, according to the Lee et. al
    correlation () for a flat plate, which is valid for: ???
               Ta = 21  [C],
        1.3 <= V  < 5.3 [m/s],
        7.5 <= w  < 15  [g/kg_a],
        -30 <= Tp < -5  [C],
        0   <= t  < 200 [min]

    ..math::
        k_f = 0.132 + 3.13*10^-4*rho_f + 1.6*10**-7 * rho_f**2

    Parameters
    ----------
    rho_f : float 
        Frost density [kg/m^3]
    
    Returns
    -------
    k_f : Frost Thermal Conductivity [J/kg*K]
    
    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] K.S. Lee, T.H. Lee, W.S. Kim, Heat and Mass Transfer of parallel plate
    heat exchanger under frosting condition, Kor. J. Air-Cond. Refrig. Eng. 6(2)
    (1994) 155-165
    '''         
    return 0.132 + 3.13*10^-4*rho_f + 1.6*10**-7 * rho_f**2


def Ostin(rho_f):
    r'''Calculates the frost thermal conductivity, k_f, according to the Ostin
    & Andersson correlation for parallel plates, which is valid for: 
        20  <= Ta <= 21    [C],
                V  = 3     [m/s],
        4.6 <= w  <= 10.50 [g/kg_a],
        -20 <= Tp <= -7    [C],
        0   <= t  <= 300   [min]

    ..math::
        k_f = -8.71*10**-3 +4.39*10**-4 * rho_f + 1.05*10**-6*rho_f**2

    Parameters
    ----------
    rho_f : float 
        Frost density [kg/m^3]
    
    Returns
    -------
    k_f : Frost Thermal Conductivity [J/kg*K]
    
    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Ostin R, Anderson S. Frost growth parameters in a forced air stream.
    Int J Heat Mass Transfer 1991;14(4/5):1009-17
    '''
    return -8.71*10**-3 +4.39*10**-4 * rho_f + 1.05*10**-6*rho_f**2


def Yonko(rho_f):
    r'''Calculates the frost thermal conductivity, k_f, according to the Yonko
    & Sepsy correlation (1967) for a flat plate, which is valid for: 
               Ta = 21  [C],
        1.3 <= V  < 5.3 [m/s],
        7.5 <= w  < 15  [g/kg_a],
        -30 <= Tp < -5  [C],
        0   <= t  < 200 [min]

    ..math::
        k_f = 0.014 + 0.00668*rho_f + 0.0001759*rho_f**2

    Parameters
    ----------
    rho_f : float 
        Frost density [kg/m^3]
    
    Returns
    -------
    k_f : Frost Thermal Conductivity [J/kg*K]
    
    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Yonko JD, Sepsy CF. An investigation of the thermal conductivity of 
    frost while forming on a flat horizontal plate. ASHRAE Transactions 1967;73(2):1.1-1.11
    '''
    return 0.014 + 0.00668*rho_f + 0.0001759*rho_f**2


# -*- coding: utf-8 -*-
"""
.. contents:: :local:

Frost Thickness
---------------
.. autofunction:: Cremers_free
.. autofunction:: Cremers
.. autofunction:: Hermes_2012
.. autofunction:: Lee
.. autofunction:: Okoroafor
.. autofunction:: Schneider
.. autofunction:: Sengupta
.. autofunction:: Yang
"""

__all__ = ['Cremers_free', 'Cremers', 'Hermes_2012', 'Lee', 'Okoroafor', 
           'Schneider', 'Sengupta', 'Yang']

def Cremers_free(Tf, Tp, t):
    r'''Calculates the frost thickness, z, according to the Cremers & Mehra correlation
    for a tube, which is valid for: 
        Ta = 24 [C]
        Free convection
        9.2 <= w   <= 15 [g/kg_a],
        45  <= psi <= 90 [%], 

    ..math::
        y_f = 0.20*t*(Tf - Tp)**0.4

    Parameters
    ----------
    Tf : float 
        Frost Temperature [K]
    Tp : float
        Cold Plate Surface [K]
    t : float
        time [s]
    
    Returns
    -------
    y_f : Frost Thickness [m]
    
    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''    
    return 0.20*t*(Tf - Tp)**0.4


def Cremers(Tf, Tp, t):
    r'''Calculates the frost thickness according to the Cremers & Mehra correlation
    for a tube, which is valid for: 
        -90 <= Tp  <= -60 [C],
        0   <= t   <= 600 [min],
        34  <= psi <= 53  [%], 

    ..math::
        y_f = 0.12*t*(Tf - Tp)**0.43

    Parameters
    ----------
    Tf : float 
        Frost Temperature [K]
    Tp : float
        Cold Plate Surface [K]
    t : float
        time [s]
    
    Returns
    -------
    y_f : Frost Thickness [m]
    
    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Cremers CJ, Mehra VK. Frost formation on vertical cylinders in free
    convection. ASME Journal of Heat Transfer 1908;104(1):3-7

    '''    
    return 0.12*t*(Tf - Tp)**0.43


def Hermes_2012():
    r'''Calculates the frost thickness according to Hermes et. al (2012) correlation
    for a flat plate, which is valid for:
        5.3    <= a1*(Ta-Tw) <= 8.5    [C]
        0.0057 <=   wa-ww    <= 0.0090 [kg_v/kg_a]
                      Re     <= 3*10^7 [-]
        29.3   <=     Nu     <= 40.6   [-]
        82     <=   rho_f    <= 318    [kg/m^3]
        0      <=      t     <= 120    [min]
        
    ..math:

    Parameters
    ----------
    Ta : float
        Air Temperature [C]
    Tw : float
        Plate Temperature [C]
    Tm : float
        Melting-point Temperature of water-ice [C]
    Tf : float
        Frost Temperature [C]
    pv : float
        Partial vapor pressure of the air [-]        
    psat : float
        Vapor pressure of saturated air [%]
    psatf : float
        Vapor pressure of saturated air at the frost surface temperature [-]
    kice : float
        Ice thermal conductivity (kJ/kg*K)
    pice : float
        Ice density (kg/m^3)
    isv : float
        
    t : float
        Time [s]
    
    Returns
    -------
    rho : Frost density [kg/m^3]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Hermes, C.J., 2012. An analytical solution to the problem of frost growth
    and densification on flat surfaces. Int. J. Heat Mass Transf. 55 (23-24), 7346-7351.

    '''
    
    return 


def Lee():
    r'''Calculates the frost thickness, z, according to the Lee (1997) correlation
    for a flat plate, which is valid for:
                Tw     = -15   [C],
                Ta     = 25    [C], 
        50   <= RH    <= 80    [%],
        6000 <= Re    <= 50000 [-],
        50   <= rho_f <= 400   [kg/m^3],
        0    <= t     <= 120   [min]
        
    ..math::
        

    Parameters
    ----------
    Ta : float
        Air Temperature [C]
    Tw : float
        Plate Temperature [C]
    Tm : float
        Melting-point Temperature of water-ice [C]
    Tf : float
        Frost Temperature [C]
    pv : float
        Partial vapor pressure of the air [-]        
    psat : float
        Vapor pressure of saturated air [%]
    psatf : float
        Vapor pressure of saturated air at the frost surface temperature [-]
    kice : float
        Ice thermal conductivity (kJ/kg*K)
    pice : float
        Ice density (kg/m^3)
    isv : float
        
    t : float
        Time [s]
    
    Returns
    -------
    rho : Frost density [kg/m^3]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Lee, K.-S., Kim, W.-S. Lee, T.-H., 1997. A one-dimensional model for 
    frost formation on a cold surface. Int. J. Heat Mass Transf. 40, 4359-4365.

    '''
    return 3


def Okoroafor(b0, b1, t, n):
    return b0 + b1*t**n


def Schneider(Tf, Tm, Tw, k_i, hsub, rho_i, t, p, pfsat, psat):
    r'''Calculates the frost thickness, z, according to Schneider et. al (1978) correlation
    for a flat plate, which is valid for:
        -30  <= Tw <= -5    [C],
        5    <= Ta <= 15    [C], 
        96   <= RH <- 99    [%],
        4000 <= Re <= 32000 [-],
        60   <= t  <= 480   [min]
        
    ..math::
    

    Parameters
    ----------
    Ta : float
        Air Temperature [C]
    Tw : float
        Plate Temperature [C]
    Tm : float
        Melting-point Temperature of water-ice [C]
    Tf : float
        Frost Temperature [C]
    pv : float
        Partial vapor pressure of the air [-]        
    psat : float
        Vapor pressure of saturated air [%]
    psatf : float
        Vapor pressure of saturated air at the frost surface temperature [-]
    kice : float
        Ice thermal conductivity (kJ/kg*K)
    pice : float
        Ice density (kg/m^3)
    isv : float
        
    t : float
        Time [s]
    
    Returns
    -------
    z : Frost thickness [m]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Schneider, H.W., 1978. Equation of the growth rate of frost forming on
    cooled surfaces. Int. J. Heat Mass Trasnfer. 21 (8), 1019-1024.

    '''  
    Ft = 1 + 0.052 * ((Tf - Tm) / (Tm - Tw))
    return (0.465 * ((k_i * (Tf - Tw))/ (hsub * rho_i))**0.5 * (t/3600)**-0.03 
         * (Tf - Tm)**0.01 * ((p-pfsat) / (psat - pfsat))**0.25 * Ft)


def Sengupta(d, Re, Pr, w, Fo):
    r'''Calculates the frost thickness according to the Sengupta et al. correlation
    for a cylinder in cross-flow, which is valid for: 
                Ta = 29   [C]
        1.5  <= V <= 4.4  [m/s]
        10.0 <= w <= 20.0 [g/kg_a]
        
    ..math::
        y_f = 0.84 * d * Re**-0.15 * Pr**0.65 * (1+w)**0.71 * Fo**0.11

    Parameters
    ----------
    d : float 
        cylinder diameter [m]
    Re : float
        Reynolds number [-]
    Pr : float
        Prandtl number [-]
    w : float
        Relative Humidity [%]
    Fo : float
        Fourier number [-]
    
    Returns
    -------
    y_f : Frost Thickness [m]
    
    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Segupta S, Sherif SA, Wong KV. Empirical heat transfer and frost thickness
    correlations during frost deposition on a cylinder in cross-flow in the 
    transient regime. Int J Energy Research 1998;615-24.
    '''    
    return 0.84 * d * Re**-0.15 * Pr**0.65 * (1+w)**0.71 * Fo**0.11


def Yang(Re, Fo, w, Ta, Ttp, Tp, L):
    r'''Calculates the frost thickness according to the Yang et al. correlation
    for a flat plate, which is valid for: 
               Ta = 21  [C],
        1.3 <= V  < 5.3 [m/s],
        7.5 <= w  < 15  [g/kg_a],
        -30 <= Tp < -5  [C],
        0   <= t  < 200 [min]
        
    ..math::
        y_f = 2.878 * (Re**0.193) * (Fo**0.573) * (w**1.738) * (T**-1.029)

    Parameters
    ----------
    Re : float
        Reynolds number [-]  
    Fo : float
        Fourier number [-]
    w : float
        Absolute humidity [kg_w/kg_air]
    Ta : float
        Air temperature [C]
    Tp : float
        Plate temperature [C]
    Ttp : float
        Triple point of water [C]
    L : float
        plate length [m]
    
    Returns
    -------
    y_f : Frost Thickness [m]
    
    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Yang, D.-K., Lee, K-S., 2004. Dimensionless correlations of frost
    properties on a cold plate. Int J. Refrigeration 27 (1), 89-96.
    '''  
    Tstar = (Ta - Ttp) / (Ta - Tp)
    y_fstar = 2.878 * (Re**0.193) * (Fo**0.573) * (w**1.738) * (Tstar**-1.029)
    return y_fstar * L 


# -*- coding: utf-8 -*-
"""
.. contents:: :local:

Dimensionless Numbers
---------------------
.. autofunction:: Properites
.. autofunction:: Fourier
.. autofunction:: Hatta
.. autofunction:: Jakob
.. autofunction:: Lewis
.. autofunction:: Nusselt
.. autofunction:: Reynolds
.. autofunction:: Sherwood

Dimensions
----------
.. autofunction:: thermal_conductivity

"""

import numpy
import CoolProp as cp
import warnings


__all__ = ['Fourier', 'Hatta', 'Jakob', 'Lewis', 'Nusselt', 'Reynolds', 'Sherwood']


def Properties(T, P=101325, fluid='air'):
    r'''Calculates fluid properties of air at atmospheric conditions using CoolProp

    Parameters
    ----------
    T : float
        Temperature [K]
    P : float (optional)
        Pressure [Pa] (Default = 101325 Pa)
    fluid : string
        fluid type (Default = 'air')
    
    Returns
    -------
    rho   : Density [-]
    mu    : Dynamic Viscosity []
    nu    : Kinematic Viscosity []
    c     : Specific Heat []
    K     : Thermal Conductivity []
    alpha : Thermal Diffusivity []

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''      
    rho = cp.CoolProp.PropsSI('D', 'T', T, 'P', P, fluid) # [kg/m##3]
    mu  = cp.CoolProp.PropsSI('V', 'T', T, 'P', P, fluid) #
    nu  = mu /rho
    c_p = cp.CoolProp.PropsSI('C', 'T', T, 'P', P, fluid) #
    k   = cp.CoolProp.PropsSI('L', 'T', T, 'P', P, fluid) #
    alpha = thermal_diffusivity(k, rho, c_p)
    return rho, mu, nu, c_p, k, alpha


def Fourier(alpha_a, t, L):
    r'''Calculates the Fourier number 

    ..math::
        (a + b)^2 = a^2 + 2ab + b^2
        

    Parameters
    ----------
    alpha_a : float
        Thermal Diffusivity [m^2/s]
    t   : float
        Time [s]
    L : float
        plate length [m]
    
    Returns
    -------
    Fo : Fourier number [-]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''   
    return (alpha_a * t) / (L**2)


def Jakob(cp, i_sv, Tsat_a, T_w, omega_a, omega_sat_w):
    r'''Calculates the Jakob number 

    ..math::
        \lambda = \frac(Cp/isv) * (Tsat_a - T_w)/(omega_a - omega_sat_w)

    Parameters
    ----------
    i_sv : float
        Latent heat of sublimation [J/kg]
    Cp   : float
        Specific heat of dry air [J/kg*K] 
    Tsat_a : float
        Saturation temperature of air [C]
    T_w  : float
        Plate surface temperature [C]
    omega_a  : float
        Humidity Ratio [kg_vapor/kg_total]
    omega_sat_w : float
        Humidity ratio @ Tsat
    
    Returns
    -------
    rho : Density of frost (kg/m^3)

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''        
    return (cp/i_sv) * (Tsat_a - T_w)/(omega_a - omega_sat_w)


def Lewis(D, alpha):
    r'''Calculates the Lewis number
    ..math::
        Le = D / alpha

    Parameters
    ---------   
    D : float
        Diffusion coefficient of water-vapor in the air (m^2/s)
    alpha  : float
        Thermal diffusivity of air (m^2/s)
    
    Returns
    -------
    Ha : Hatta number (-)


    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''      
    return D/alpha


def Hatta(x_s, lamb, t, epsilon, D):
    r'''Calculates the Hatta number
    ..math::
        \rho_f = 207*exp(0.266*T_f - 0.0615*T_w)

    Parameters
    ---------   
    x_s     : float
           Frost thickness (m)
    lamb    : float
        Desublimation Coefficient (1/s)
    t       : float
        Time (s)
    epsilon : float
        Porosity (-)
    D       : float
        Diffusivity of water vapor in air (m^2/s)
    
    Returns
    -------
    Ha : Hatta number (-)


    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''  
    return x_s* numpy.sqrt((lamb * t) / (epsilon * D))
    

def Nusselt(h, L, k_a):
    r'''Calculates the Nusselt number
    ..math::
        Nu = h * L / k_a

    Parameters
    ---------   
    h   : float
        Heat transfer coefficient [W/m^2*s]
    L   : float
        Plate length [m]
    k_a : float
        Thermal conductivity of Air [W/mK]
    
    Returns
    -------
    Nu : Nusselt number (-)

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''      
    return h*L / k_a
    

def Reynolds(V, L, nu):
    r'''Calculates the Reynolds number
    ..math::
        Re = V*L/nu

    Parameters
    ---------   
    V   : float
        Air Velocity [m/s]
    L   : float
        Plate length [m]
    nu : float
        Dynamic Viscosity [m^2/s]
    
    Returns
    -------
    Re : Reynolds number (-)

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    ''' 
    return (V*L) / nu
    

def Sherwood(h_m, L, rho_a, D_a):
    r'''Calculates the Sherwood number
    ..math::
        Sh = h_m * L / rho_a * D_a

    Parameters
    ---------   
    h_m  : float
        Mass transfer coefficient [kg/m^2*s]
    L    : float
        Plate length [m]
    rho_a  : float
        Density of Air [kg/m^3]
    D_a : float
        Mass Diffusionn Coefficient [m^2/s]
    
    Returns
    -------
    Sh : Sherwood number (-)


    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''  
    return (h_m * L) / (rho_a * D_a)

def dimensionless_time(k, t, rho, Cp, l):
    r'''Calculates the dimensionless time constant
    ..math::
        \tau = (k*t) / (rho*Cp*l**2)

    Parameters
    ---------   
    k  : float
        Thermal conductivity [W/m^2*K]
    t   : float
        Time [s]
    rho : float
        Density [kg/m^3]
    Cp : float
        Specific heat [J/kg*K]
    l  : float
        Plate length [m]
    
    Returns
    -------
    tau : dimensionless time constant (-)


    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''      
    return (k*t) / (rho*Cp*l**2)


def dimensionless_temperature(T_a, T_w):
    r'''Calculates the dimensionless temperature
    ..math::
        

    Parameters
    ---------   
    T_a  : float
        Air Temperature [K]
    T_w  : float
        Wall Temperature [K]

    
    Returns
    -------
    T : dimensionless temperature (-)


    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    ''' 
    
    T_tp = 273.16 # triple point of water [K]
    return (T_a - T_tp) / (T_a - T_w)

# Dimensional
def thermal_diffusivity(k, rho, Cp):
    r'''Calculates thermal diffusivity or `alpha` for a fluid with the given
    parameters.

    .. math::
        \alpha = \frac{k}{\rho Cp}

    Parameters
    ----------
    k : float
        Thermal conductivity, [W/m/K]
    rho : float
        Density, [kg/m^3]
    Cp : float
        Heat capacity, [J/kg/K]

    Returns
    -------
    alpha : float
        Thermal diffusivity, [m^2/s]

    Notes
    -----

    Examples
    --------
    >>> thermal_diffusivity(k=0.02, rho=1., Cp=1000.)
    2e-05

    References
    ----------
    .. [1] Blevins, Robert D. Applied Fluid Dynamics Handbook. New York, N.Y.:
       Van Nostrand Reinhold Co., 1984.
    '''
    return k/(rho*Cp)


def saturation_pressure(T, fluid = 'air'):
    p_sat = cp.CoolProp.PropsSI('P', 'Q', 0, 'T', T, fluid)
    return p_sat


def partial_pressure(RH, p_sat):
    return (RH/100)*p_sat


def humidity_ratio(p_va, p_atm = 101325):
    return (0.62198*p_va)/(p_atm - p_va)


def validity_range_check(t_w = 0, t_w_low = 0, t_w_upp = 0,
                         t_a = 0, t_a_low = 0, t_a_upp = 0,
                         w_a = 0, w_a_low = 0, w_a_upp = 0,
                         v = 0, v_low = 0, v_upp = 0,
                         rho_f = 0, rho_f_low = 0, rho_f_upp = 0,
                         rho_i = 0, rho_i_low = 0, rho_i_upp = 0,
                         rho_a = 0, rho_a_low = 0, rho_a_upp = 0):

    '''
    Checks the validity range of parameters according to inputs
    '''
    
    if(t_w != 0 and t_w < t_w_low or t_w > t_w_upp):
        warnings.warn('t_w is extrapoled')
    elif(t_a != 0 and t_a < t_a_low or t_a > t_a_upp):
        warnings.warn('t_a is extrapoled')
    elif(w_a != 0 and w_a < w_a_low or w_a > w_a_upp):
        warnings.warn('w_a is extrapoled')
    elif(v != 0 and v < v_low or v > v_upp):
        warnings.warn('v is extrapoled')
    elif(rho_f != 0 and rho_f < rho_f_low or rho_f > rho_f_upp):
        warnings.warn('rho_f is outside range')
    elif(rho_i != 0 and rho_i < rho_i_low or rho_i > rho_i_upp):
        warnings.warn('rho_i is outside range')
    elif(rho_a != 0 and rho_a < rho_a_low or rho_a > rho_a_upp):
        warnings.warn('rho_a is outside range')


# -*- coding: utf-8 -*-
"""
.. contents:: :local:

Density Correlations
--------------------
.. autofunction:: Hayashi
.. autofunction:: modified_Hayashi
.. autofunction:: Hermes_2009
.. autofunction:: Hermes_2014
.. autofunction:: Hosoda
.. autofunction:: Kandula
.. autofunction:: Leoni
.. autofunction:: Mao
.. autofunction:: Sommers
.. autofunction:: Yang
"""
import numpy

__all__ = ['Hayashi', 'modified_Hayashi', 'Hermes_2009', 'Hermes_2014', 'Hosoda',
           'Kandula', 'Leoni', 'Sommers', 'Yang']

def Hayashi(Tf):    
    r'''Calculates the frost density, rho_f, according to the Hayashi correlation (1977)
    for a flat plate, which is valid for: 
        -15 <= Tw  <= -5  [C],
        16  <= Ta  <= 22  [C], 
        0.5 <= psi <= 0.8 [-],
               u_a  = 0.7 [m/s]

    ..math::
        \rho_f = 207*exp(0.266*T_f - 0.0615*T_w)

    Parameters
    ----------
    Tf : float 
        Frost surface temperature [C]
    
    Returns
    -------
    rho_f : Frost density (kg/m**3)

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Hayashi, Y., Aoki, A., Hori, K., 1977. Study of frost properties correlating
    with frost formation types. J. Heat Transfer 99, 239-245.
    '''
    return 650*numpy.exp(0.277*Tf)


def modified_Hayashi(Tf, Tw):
    r'''Calculates the frost density, rho_f, according to the modified Hayashi
    correlation () for a flat plate, which is valid for: 
        -15 <= Tw  <= -5  [C],
        16  <= Ta  <= 22  [C], 
        0.5 <= psi <= 0.8 [-],
                u_a = 0.7 [m/s]

    ..math::
        \rho_f = 207*exp(0.266*T_f - 0.0615*T_w)

    Parameters
    ----------
    Tf : float 
        Frost surface temperature [C]
    Tw : float
        Plate surface temperature [C]
    
    Returns
    -------
    rho : Frost density (kg/m**3)

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''
    return numpy.exp(0.266*Tf - 0.0615*Tw)


def Hermes_2009(lamb, t):
    r'''Calculates the density of frost according to the Hermes correlation
    for a flat plate, which is valid for: 
        -15 < Tw < -5 [C],
        16 < Ta < 22 [C], 
        0.5 < psi < 0.8 [-],
        u_a = 0.7 [m/s]

    ..math::
        \rho_f = 207*exp(0.266*T_f - 0.0615*T_w)

    Parameters
    ----------
    lamb : float 
        modified Jakob number [-]
    t : float
        time [s]
    
    Returns
    -------
    rho : Density of frost (kg/m**3)

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Hermes, C.J.L., Piucco, R.O., Melo, C., Barbosa Jr., J.R., 2009. A
    study of frost growth and densification on flat surfaces. Exp. Therm. Fluid
    Sci. 33, 371-379.

    '''
    return 2.2 * (lamb ** (3/2)) * numpy.sqrt(t)


def Hermes_2014(Cp, Tsata, Tw, hsub, wa, wsatw, t):
    r'''Calculates the density of frost according to the Hermes correlation
    for a flat plate, which is valid for: 
        -15 < Tw < -5 [C],
        16 < Ta < 22 [C], 
        0.05 < psi < 0.8 [%],
        0 < t < 120 [min]
        u_a = 0.7 [m/s]

    ..math::
        \rho_f = 2.2 * ((Cp*(Tsata - Tw)) / (hsub*(wa - wsatw)))**-1.5* t**0.5

    Parameters
    ----------
    Cp : float 
        Specific Heat [J/kg*K]
    Tsata : float
        Saturation Temperature of Air [K]
    Tw : float
        
    hsub : float
        
    wa : float
        Relative Humidity of air [%]
    wsatw : float
        [%]
    t : float
        time [s]
    
    Returns
    -------
    rho : Frost density [kg/m**3]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Hermes, C. A semi-empirical correlation for the frost density

    '''    
    return 2.2 * ((Cp*(Tsata - Tw)) / (hsub*(wa - wsatw)))**-1.5* t**0.5


def Hosoda(Tw, Va):
    r'''Calculates the frost density according to the Hosoda and Uzuhashi correlation
    for a flat plate, which is valid for: 
        -15 < Tw < -5 [C],
       
    ..math::
        \rho_f = 340*|Tw|**-0.445 + 85*Va

    Parameters
    ----------
    Va : float 
        Air Velocity [m/s]
    Tw : float
        Plate surface temperature [C]
    
    Returns
    -------
    rho_f : Frost density [kg/m**3]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Hosoda T, Uzuhashi H. Effects of frost on the heat transfer coefficient.
    Hitachi Reivew 1967;16(6):254-9.
    '''    
    return 340*numpy.absolute(Tw)**-0.445 + 85*Va


def Kandula(rho_i, Tf, Tw, Tm, Re, Rec):
    r'''Calculates frost density according to the Kandula correlation (2012)
    for a flat plate, which is valid for: 
        -20 < Tw < -5  [C],
        10  < Ta < 22  [C], 
        0.7 < v  < 2.5 [m/s],
        50  < RH < 80  [%]
        
    ..math::
        \rho_f = (rho_i * (0.5*((Tf-Tw)/(Tm-Tw)) 
                     * numpy.exp(-(0.375+1.5*((Tf-Tw)/(Tm-Tw)))) 
                     * (1 - numpy.sqrt(Re/Rec))) )

    Parameters
    ----------
    rho_i : float 
        Ice Density [kg/m^3]
    Tf : float
        Temperature of frost  [K]     
    Tw : float
        Temperature of vapor [K]
    Tm : float
        Temperature of  [K]        
    Re : float
        Reynolds number [-]
    Rec : float
        Critical Reynolds number [-]
        
    Returns
    -------
    rho_f : Frost density [kg/m^3]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Kandula, M., 2011. Correlation of water frost porosity in laminar 
    flow over flat surfaces. Special Top. Rev. Porous Media - Int. J. 3, 79-87
    '''    
    return (rho_i * (0.5*((Tf-Tw)/(Tm-Tw)) 
                     * numpy.exp(-(0.375+1.5*((Tf-Tw)/(Tm-Tw)))) 
                     * (1 - numpy.sqrt(Re/Rec))) )
    

def Leoni(Re, Ja, ww, wa, t):
    r'''Calculates the density of frost according to the Leoni et. al (2020) correlation
    for a flat plate, which is valid for: ???
        -35 < Tw < -15 [C],
        -5 < Ta < 15 [C], 
        1.0 < v < 2.5 [m/s]
        0.00322 < w < 0.00847 [kg_v/kg_a]
        
    ..math::
        \rho_f = 5.47*Re**0.16 * Ja**0.29 * (ww/wa)**0.61 * t**0.34

    Parameters
    ----------
    Re : float
        Reynolds number [-]
    Ja : float
        Jakob number [-]        
    ww : float
        Relative Humidity of [%]
    wa : float
        Relative Humidity of air [%]
    t : float
        Time [s]
    
    Returns
    -------
    rho : Frost density [kg/m^3]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''    
    return 5.47*Re**0.16 * Ja**0.29 * (ww/wa)**0.61 * t**0.34
  
    
def Mao(Re):
    r'''Calculates the frost density, rho_f, according to the Mao et. al (1992) correlation
    for a flat plate, which is valid for:
        -15  < Tw < -15  [C],
        15   < Ta < 23   [C], 
        1.15 < v  < 2.67 [m/s]
        
    ..math::
        

    Parameters
    ----------
    Re : float
        Reynolds number [-]
    Ja : float
        Jakob number [-]        
    ww : float
        Relative Humidity of [%]
    wa : float
        Relative Humidity of air [%]
    t : float
        Time [s]
    
    Returns
    -------
    rho_f : Frost density [kg/m^3]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Mao, Y., Besant, R.W., Rezkallah, K.S., 1992. Measurement and correlations
    of frost properties with airflow over a flat plate. ASHRAE Trans. 98, 65-77.
    '''     
    return 1

    
def Sommers(lamb, sigma, RH, t):
    r'''Calculates the density of frost according to the Sommers et al. correlation
    for a flat plate, which is valid for: 
        -15 < Tw < -5 [C],
        16 < Ta < 22 [C], 
        0.5 < psi < 0.8 [-],
        u_a = 0.7 [m/s]

    ..math::
        \rho_f = (2.16*lamb**(-3/2) * t**0.5 * (45/numpy.log(sigma))**0.95 * (0.8/RH)**0.60)

    Parameters
    ----------
    lamb : float 
        modified Jakob number [-]
    sigma : float
        Contact Angle [degrees]
    RH : float
        relative humidity [%]
    t : float
        time [s]
    
    Returns
    -------
    rho : Frost density (kg/m**3)

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    '''    
    return (2.16*lamb**(-3/2) * t**0.5 * (45/numpy.log(sigma))**0.95 * (0.8/RH)**0.60)


    
def Yang(rho_i, Re, Fo, wa, Ta, Ttp, Tw):
    r'''Calculates the density of frost according to the Yang and Lee correlation
    for a flat plate, which is valid for: 
        -35     < Tw < -15     [C],
        -5      < Ta < 15      [C], 
        1.0     < v  < 2.5     [m/s],
        0.00322 < w  < 0.00847 [kg_v/kg_a]
        
    ..math::
        \rho_f = 2.2 * ((Cp*(Tsata - Tw)) / (hsub*(wa - wsatw)))**-1.5* t**0.5

    Parameters
    ----------
    rho_i : float 
        Ice Density [kg/m^3]
    Re : float
        Reynolds number [-]
    Fo : float
        Fourier number [-]        
    wa : float
        Relative Humidity of air [%]
    Ta : float
        Temperature of air  [K]
    Ttp : float
        Triple-point of water
    Tw : float
        Temperature of vapor [K]
    
    Returns
    -------
    rho : Frost density [kg/m**3]

    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] Yang, D.-K., Lee, K-S., 2004. Dimensionless correlations of frost
    properties on a cold plate. Int J. Refrigeration 27 (1), 89-96.
    '''
    return (rho_i * (1.54*10**-4 * Re**0.351 * Fo**0.311 * wa**-0.413 * 
                     (Ta-Ttp)/(Ta-Tw)))


# -*- coding: utf-8 -*-
"""
.. contents:: :local:

Porosity
---------------
.. autofunction:: Hermes

"""
from frostypy import core
import numpy as np

__all__ = ['Hermes']

# Frost Porosity
def Hermes(rho_f, rho_i, rho_a):
    r'''Calculates the frost thickness according to the Cremers & Mehra correlation
    for a tube, which is valid for: 
        Ta = 24 [C]
        Free convection
        9.2 < w < 15 [g/kg_a],
        45 < psi < 90 [%], 

    ..math::
        y_f = 0.20*t*(Tf - Tp)**0.4

    Parameters
    ----------
    Tf : float 
        Frost Temperature [K]
    Tp : float
        Cold Plate Surface [K]
    t : float
        time [s]
    
    Returns
    -------
    y_f : Frost Thickness [m]
    
    Examples
    --------
    >>>
    
    References
    ----------
    ..[1] (insert here)

    ''' 
    
    # Check parameters for validity
    core.validity_range_check(rho_f = rho_f,
                                       rho_f_low = 10,
                                       rho_f_upp = 100)


    # Correlation
    return (rho_f - rho_i)/(rho_a - rho_i)
